@@ -223,18 +223,24 @@ class DispatcherExecutor(Executor):
         for par in template.inputs.parameters.values():
             if par.save_as_artifact:
                 self.task_dict["forward_files"].append("./" + par.path)
+        merge = self.merge_sliced_step and hasattr(template, "slices") and\
+            template.slices is not None
+        if merge:
+            sliced_output_parameters = template.slices.output_parameter.copy()
+            if "dflow_success_tag" in template.outputs.parameters:
+                sliced_output_parameters.append("dflow_success_tag")
         self.task_dict["backward_files"] = []
         for art in template.outputs.artifacts.values():
             self.task_dict["backward_files"].append("./" + art.path)
-        for par in template.outputs.parameters.values():
-            if par.name == "dflow_success_tag":
-                continue
+        for name, par in template.outputs.parameters.items():
             if par.save_as_artifact:
                 self.task_dict["backward_files"].append(
                     "./" + par.value_from_path)
             elif par.value_from_path is not None:
                 self.task_dict["backward_files"].append(
                     "./" + par.value_from_path)
+        if merge and len(sliced_output_parameters) > 0:
+            self.task_dict["backward_files"] = ["./tmp/outputs"]
 
         new_template.script = "import os\n"
         new_template.script += "os.chdir('%s')\n" % self.work_root
@@ -276,8 +282,7 @@ class DispatcherExecutor(Executor):
         new_template.script += "for f in task.backward_files:\n"
         new_template.script += "    os.makedirs(os.path.dirname(f), "\
             "exist_ok=True)\n"
-        if self.merge_sliced_step and hasattr(new_template, "slices") and \
-                new_template.slices is not None:
+        if merge:
             new_template.inputs.parameters["dflow_with_param"] = \
                 InputParameter(value="")
             new_template.inputs.parameters["dflow_sequence_start"] = \
@@ -324,6 +329,14 @@ class DispatcherExecutor(Executor):
                     new_template.script += "    new_script = new_script."\
                         "replace(%s, %s if isinstance(%s, str) else "\
                         "json.dumps(%s))\n" % (old, new, new, new)
+            for name in template.slices.output_parameter:
+                new_template.script += "    new_script = new_script.replace("\
+                    "\"handle_output_parameter('%s'\", "\
+                    "\"handle_output_parameter('%s_\" + str(i) + \"'\")\n" % (
+                        name, name)
+            if "dflow_success_tag" in template.outputs.parameters:
+                new_template.script += "    new_script = new_script.replace("\
+                    "'success_tag', 'success_tag_' + str(i))\n"
             new_template.script += "    with open('script' + str(i), 'w')"\
                 " as f:\n"
             new_template.script += "        f.write(new_script)\n"
@@ -353,6 +366,17 @@ class DispatcherExecutor(Executor):
             new_template.script += "        time.sleep(2**retry)\n"
         else:
             new_template.script += "submission.run_submission()\n"
+        if merge:
+            for name in sliced_output_parameters:
+                path = template.outputs.parameters[name].value_from_path
+                new_template.script += "res = []\n"
+                new_template.script += "for i in range(len(item_list)):\n"
+                new_template.script += "    fname = './%s_' + str(i)\n" % path
+                new_template.script += "    if os.path.isfile(fname):\n"
+                new_template.script += "        with open(fname, 'r') as f:\n"
+                new_template.script += "            res.append(f.read())\n"
+                new_template.script += "with open('./%s', 'w') as f:\n" % path
+                new_template.script += "    f.write(json.dumps(res))\n"
 
         # workaround for unavailable exit code of Bohrium job
         # check output files explicitly
